import { consola, createConsola } from 'consola';

export declare function logLevel(): number;
export declare const logger: Log;
declare type ErrorMessage = string
export type ErrorOptions =
  | {
    shouldExit: boolean
    silent?: boolean
    message?: ErrorMessage
  }
  | any
  | Error

export interface Log {
  info: (...args: any[]) => void
  success: (msg: string) => void
  error: (err: string | Error | object | unknown, options?: ErrorOptions) => void
  warn: (arg: string) => void
  warning: (arg: string) => void
  debug: (...args: any[]) => void
  dump: (...args: any[]) => void
  dd: (...args: any[]) => void
  echo: (...args: any[]) => void
}

export interface LogOptions {
  styled?: boolean
}

export const log: Log = {
  info: async (message: string, options?: LogOptions) => {
    if (options?.styled === false)
      console.log(message)
    else logger.info(message)
    await writeToLogFile(`INFO: ${stripAnsi(message)}`)
  },

  success: async (message: string, options?: LogOptions) => {
    if (options?.styled === false)
      console.log(message)
    else logger.success(message)
    await writeToLogFile(`SUCCESS: ${stripAnsi(message)}`)
  },

  warn: async (message: string, options?: LogOptions) => {
    if (options?.styled === false)
      console.log(message)
    else logger.warn(message)
    await writeToLogFile(`WARN: ${stripAnsi(message)}`)
  },

  warning: async (message: string, options?: LogOptions) => {
    if (options?.styled === false)
      console.log(message)
    else logger.warn(message)
    await writeToLogFile(`WARN: ${stripAnsi(message)}`)
  },

  error: async (err: string | Error | object | unknown, options?: ErrorOptions) => {
    handleError(err, options)
  },

  debug: async (...args: any[]) => {
    const formattedArgs = args.map(arg => (typeof arg === 'object' ? JSON.stringify(arg, null, 2) : arg))
    const message = `DEBUG: ${formattedArgs.join(' ')}`

    if (process.env.APP_ENV === 'production' || process.env.APP_ENV === 'prod')
      return writeToLogFile(message)

    logger.debug(message)
    await writeToLogFile(stripAnsi(message))
  },

  dump: (...args: any[]) => args.forEach(arg => console.log(arg)),
  dd: (...args: any[]) => {
    args.forEach(arg => console.log(arg))
    process.exit(ExitCode.FatalError)
  },
  echo: (...args: any[]) => console.log(...args),
}
export declare function dump(...args: any[]): void;
export declare function dd(...args: any[]): void;
export declare function echo(...args: any[]): void;

export { consola, createConsola }